/*****
 -- -- -- -- -- -- -- -- -- -- Quiz settings -- -- -- -- -- -- -- -- -- --
 *****/

// ----- Consts -----
const questionTypeEnum = {
  one: "one", //radio zadatak s jednim točnim odgovorom
  multi: "multi", //checkbox zadatak s više točnih odgovora
  wordSelectMatch: "wordSelectMatch", // zadatak za uparivanje odgovora u 2 select stupca,
  wordWrite: "wordWrite", //zadatak s upisivanjem teksta u input polje
  wordSelect: "wordSelect", //zadatak za odabir ponuđenih riječi iz select elementa,
  wordDrag: "wordDrag", //zadatak s dovlačenjem riječi na odgovarajuće mjesto
  wordGroup: "wordGroup", //zadatak s dovlačenjem riječi u odgovarajuću grupu
  sortList: "sortList", // zadatak sa sortiranjem elemenata u poredak
  pairList: "pairList", // zadatak sa uparivanjem elemenata
};

const questionPrefix = 'zadatak';
const answerPrefix = 'o';
const idSeparator = '-';
const answerSeparator = ';';

const questionTypeAttribute = "data-tasktype";
const correctAnswerAttribute = "data-correct";
const dropTargetAttribute = "data-element-dragged";
const draggedTargetAttribute = "data-dragged-group-value";
const sortablePositionAttribute = "data-original-position";

/*****
 -- -- -- -- -- -- -- -- -- -- Quiz elements validation -- -- -- -- -- -- -- -- -- --
 *****/

// ----- Main -----
const checkQuiz = (quizId) => {
  const questionElementList = getQuestionElementListForQuiz(quizId);
  const questionsTotal = questionElementList.length;
  let answeredCorrectTotal = 0;

  questionElementList.forEach((questionElement) => {
    const isAnsweredCorrectly = validateQuestion(questionElement);
    if (isAnsweredCorrectly) {
      answeredCorrectTotal++;
    }
    signalizeQuestionCorrectness(questionElement, isAnsweredCorrectly);
  });
  setQuizResult(quizId, questionsTotal, answeredCorrectTotal);
};

const resetQuiz = (quizId) => {
  const questionElementList = getQuestionElementListForQuiz(quizId);

  questionElementList.forEach((questionElement) => {
    removeQuestionCorectnessSignalization(questionElement);
    resetQuestionAnswers(questionElement);
  });

  resetQuizResult(quizId);
};

const checkQuestion = (questionElementId) => {
  const questionElement = getQuestionElement(questionElementId);

  removeQuestionCorectnessSignalization(questionElement);

  const isAnsweredCorrectly = validateQuestion(questionElement);
  signalizeQuestionCorrectness(questionElement, isAnsweredCorrectly);
};

const resetQuestion = (questionElementId) => {
  const questionElement = getQuestionElement(questionElementId);

  removeQuestionCorectnessSignalization(questionElement);
  resetQuestionAnswers(questionElement);

};

// ----- Utils -----

const getQuestionElement = (questionElementId) => {
  const questionElement = $("div[id=" + questionElementId + "]").get(0);

  return questionElement;
};

const getQuestionElementListForQuiz = (quizId) => {
  const questionElementList = $('#' + quizId + ' .zadatak').toArray();

  return questionElementList;
};

const getAnswerElementListForQuestion = (answerIdPrefix, elementType) => {

  const answerElementList = $(elementType + "[id*=" + answerIdPrefix + "]").toArray();
  return answerElementList;
};

const getAnswerElementsIdPrefixForQuestion = (questionElement) => {
  //const quizId = questionElementId.substring(0, questionElementId.indexOf(idSeparator));
  //const questionId = questionElementId.substring(questionElementId.lastIndexOf(idSeparator) + 1);

  const answerIdPrefix = questionElement.id + idSeparator + answerPrefix;
  return answerIdPrefix;
};

const getQuizResultElement = (quizId) => {
  const quizResultElement = $('#' + quizId + ' .kviz-rezultat');

  return quizResultElement;
};

const getQuizResultCheckButtonElement = (quizId) => {
  const quizResultCheckButtonElement = $('#' + quizId + ' .kviz-provjeri');

  return quizResultCheckButtonElement;
};

const getQuizResultPercentageElement = (quizId) => {
  const quizResultPercentageElement = $('#' + quizId + ' .kviz-postotak');

  return quizResultPercentageElement;
};

const getQuizResultQuestionNumberElement = (quizId) => {
  const quizResultQuestionNumberElement = $('#' + quizId + ' .kviz-broj_pitanja');

  return quizResultQuestionNumberElement;
};

const getQuestionHeaderElement = (questionElement) => {
  const questionHeaderElement = $(questionElement).find('h3');

  return questionHeaderElement;
};

const getQuestionResultMarkElements = (questionElement) => {
  const questionResultMarkElements = $(questionElement).find('.correctAnswer, .incorrectAnswer');

  return questionResultMarkElements;
};

const getQuestionSortListElement = (questionElementId) => {
  const idSelector = questionElementId ? '#' + questionElementId : '';
  const sortListElement = $(idSelector + ' .sortLista');
  for (let i = 0; i < sortListElement.length; i++) {
    for (let j = 0; j < sortListElement[i].children.length; j++) {
      sortListElement[i].children[j].setAttribute(dropTargetAttribute, j);
    }
  }
  return sortListElement;
};

// ----- Answer validation  -----

const validateQuestion = (questionElement) => {
  const answeredCorrectMap = getQuestionValidation(questionElement);

  const isAnsweredCorrectly = isQuestionAnsweredCompletelyCorrect(answeredCorrectMap);

  return isAnsweredCorrectly;
};

const addImagesToAnswers = (answer, inputElement, dragNDropElement, selectElement, labelElement, listElement, image) => {
  if (labelElement) {
    labelElement.style.display = 'inline';
    answer.answer.parentElement.style.margin = '0.7em 0em';
    if (answer.answer.type === 'checkbox') {
      if ($(answer.answer)[0].checked) {
        $(labelElement).after(image);
      }
      // if ($(answer.answer).attr('data-correct') === 'true' && !$(answer.answer)[0].checked) {
      //   $(labelElement).after(image);
      // }
    } else {
      if ($(answer.answer)[0].checked) {
        $(labelElement).after(image);
      }
    }
  } else if (['OL', 'UL', 'LI'].includes(listElement)) {
    $(answer.answer).append(image);

    if (
      ['sortList', 'pairList'].includes($(answer.answer.parentElement).closest('.zadatak')[0].getAttribute('data-tasktype')) &&
      answer.answer.getAttribute('data-correct') === answer.answer.getAttribute('data-element-dragged')
    ) {
      answer.answer.classList.add('draggable-disabled');
    }
  } else if (selectElement) {
    $(selectElement).after(image);
    return;
  } else if (inputElement) {
    $(image).appendTo(inputElement);
    return;
  } else if (dragNDropElement) {
    if ($(dragNDropElement).children('div').html() !== undefined && $(dragNDropElement).children('div').html().trim().length === 0) {
      $(dragNDropElement).children('div').remove();
    }
    $(dragNDropElement).css({
      padding: '0.3em'
    });
    $(dragNDropElement).append(image);
  } else {
    if (inputElement) {
      $(answer.answer).after(image);
    }
  }
};

const getQuestionValidation = (questionElement) => {
  const questionType = questionElement.getAttribute(questionTypeAttribute);
  let answeredCorrectMap = [];
  let answerElements = [];
  const answerIdPrefix = getAnswerElementsIdPrefixForQuestion(questionElement);

  if (questionType === questionTypeEnum.one || questionType === questionTypeEnum.multi) {
    answerElements = getAnswerElementListForQuestion(answerIdPrefix, 'input');
    answeredCorrectMap = getAnswersValidationMap(answerElements, isCheckElementAnsweredCorrectly);
  } else if (questionType === questionTypeEnum.wordWrite) {
    answerElements = getAnswerElementListForQuestion(answerIdPrefix, 'input');
    answeredCorrectMap = getAnswersValidationMap(answerElements, isTextElementAnsweredCorrectly);
  } else if (questionType === questionTypeEnum.wordSelect || questionType === questionTypeEnum.wordSelectMatch) {
    answerElements = getAnswerElementListForQuestion(answerIdPrefix, 'select');
    answeredCorrectMap = getAnswersValidationMap(answerElements, isSelectElementAnsweredCorrectly);
  } else if (questionType === questionTypeEnum.wordDrag || questionType === questionTypeEnum.wordGroup) {
    answerElements = getAnswerElementListForQuestion(answerIdPrefix, 'div');
    answeredCorrectMap = getAnswersValidationMap(answerElements, isDragTextElementAnsweredCorrectly);
  } else if (questionType === questionTypeEnum.sortList || questionType === questionTypeEnum.pairList) {
    answerElements = getAnswerElementListForQuestion(answerIdPrefix, 'li');
    answeredCorrectMap = getAnswersValidationMap(answerElements, isSortListElementAnsweredCorrectly);
  }

  return answeredCorrectMap;
};

const isQuestionAnsweredCompletelyCorrect = (answeredCorrectMap) => {
  const isAllCorrect = answeredCorrectMap.every((answer) => answer.correct === true);

  return isAllCorrect;
};

const getAnswersValidationMap = (answerElements, validationFunction) => {
  const answeredCorrectMap = answerElements.map((answerElement) => {
    const isCorrect = validationFunction(answerElement);
    return {
      answer: answerElement,
      correct: isCorrect,
    };
  });
  return answeredCorrectMap;
};

//----- Text input questions
const isTextElementAnsweredCorrectly = (textElement) => {
  const possibleAnswers = textElement.getAttribute(correctAnswerAttribute);
  const inputValue = textElement.value;

  const isCorrectText = possibleAnswers.split(answerSeparator).some((answer) => answer.trim() === inputValue);
  return isCorrectText;
};

//----- Checkbox/Radio questions
const isCheckElementAnsweredCorrectly = (checkElement) => {
  const isCorrectChecked = JSON.parse(checkElement.getAttribute(correctAnswerAttribute)) === checkElement.checked; //booleans are valid JSON values

  return isCorrectChecked;
};

//----- Select questions
const isSelectElementAnsweredCorrectly = (selectElement) => {
  const isCorrectSelected = selectElement.getAttribute(correctAnswerAttribute) === selectElement.value;

  return isCorrectSelected;
};

//----- Drag text questions
const isDragTextElementAnsweredCorrectly = (dragElement) => {
  const isCorrectDragged = dragElement.getAttribute(correctAnswerAttribute) === dragElement.getAttribute(dropTargetAttribute);

  return isCorrectDragged;
};

//----- sort list questions
const isSortListElementAnsweredCorrectly = (sortElement) => {
  const isCorrectSorted = sortElement.getAttribute(correctAnswerAttribute) === sortElement.getAttribute(dropTargetAttribute);

  return isCorrectSorted;
};


/*****
 -- -- -- -- -- -- -- -- -- -- Quiz elements manipulation -- -- -- -- -- -- -- -- -- --
 *****/

const resetQuestionAnswers = (questionElement) => {
  const questionType = questionElement.getAttribute(questionTypeAttribute);
  let answerElements = [];
  const answerIdPrefix = getAnswerElementsIdPrefixForQuestion(questionElement);

  if (questionType === questionTypeEnum.one || questionType === questionTypeEnum.multi) {
    answerElements = getAnswerElementListForQuestion(answerIdPrefix, 'input');
    answerElements.forEach((answer) => {
      answer.checked = false;
    });
  } else if (questionType === questionTypeEnum.wordWrite) {
    answerElements = getAnswerElementListForQuestion(answerIdPrefix, 'input');
    answerElements.forEach((answer) => {
      answer.value = '';
    });
  } else if (questionType === questionTypeEnum.wordSelect || questionType === questionTypeEnum.wordSelectMatch) {
    answerElements = getAnswerElementListForQuestion(answerIdPrefix, 'select');
    answerElements.forEach((answer) => {
      answer.value = undefined;
    });
  } else if (questionType === questionTypeEnum.wordDrag || questionType === questionTypeEnum.wordGroup) {
    answerElements = getAnswerElementListForQuestion(answerIdPrefix, 'div');
    answerElements.forEach((answer) => {
      answer.removeAttribute(dropTargetAttribute);
      answer.removeAttribute(draggedTargetAttribute);
      $(answer).css({
        left: 0,
        top: 0,
        padding: "0.2em calc(0.3em + 14.5px)"
      }).detach().appendTo($(questionElement).find('.bazen-odgovora')[0]);
    });
  } else if (questionType === questionTypeEnum.sortList || questionType === questionTypeEnum.pairList) {
    answerElements = getAnswerElementListForQuestion(answerIdPrefix, 'li');

    const sortListElement = getQuestionSortListElement(questionElement.id);
    sortListElement.children().detach();

    const shuffledSortList = shuffleArray(answerElements)
    shuffledSortList.forEach(answer => {
      $(answer).appendTo(sortListElement);
    });

    setSortElementsOrderAttribute(questionElement.id);
    const draggableDisabledElements = $(questionElement).find('.draggable-disabled');
    for (let i = 0; i < draggableDisabledElements.length; i++) {
      $(draggableDisabledElements[i]).removeClass('draggable-disabled');
    }
  }
};

const resetKnowledgeAssessment = (quizType) => {
  $('#procjena-znanja').find(`div[id*='zadatak-'].zadatak`).each((index, task) => resetQuestion(task.id));

  $('#procjena-znanja').find('.kviz-postotak')[0].textContent = '';
  $('#procjena-znanja').find('.kviz-broj_pitanja')[0].textContent = '';
  $('#procjena-znanja').find('.kviz-rezultat').css({
    'display': 'none'
  });
  $('#procjena-znanja').find('.kviz-provjeri').css({
    'display': 'inline-block'
  });

  focusFirstQuestionElement(quizType);
};

const focusFirstQuestionElement = (quizType) => {
  if (quizType === 'unit') { // is in unit
    $('#procjena-znanja').find('.carousel-item.active')[0].classList.remove('active');
    $($('#procjena-znanja').find(`div[id*='zadatak-'].zadatak`)[0]).closest('.carousel-item')[0].classList.add('active');
  }

  if (quizType === 'module') { // is in module
    $('#procjena-znanja').find(`div[id*='zadatak-'].zadatak`)[0].scrollIntoView(false);
  }
};

const setSortElementsOrderAttribute = (questionElementId) => {
  const sortableElements = getQuestionSortListElement(questionElementId).find('li');
  sortableElements.each((index, element) => {
    element.setAttribute(dropTargetAttribute, index);
  });
};

// ----- Utils -----

const setQuizResult = (quizId, questionsTotal, answeredCorrectTotal) => {
  const quizResultElement = getQuizResultElement(quizId);
  const quizResultCheckButtonElement = getQuizResultCheckButtonElement(quizId);
  const quizResultPercentageElement = getQuizResultPercentageElement(quizId);
  const quizResultQuestionNumberElement = getQuizResultQuestionNumberElement(quizId);

  let percentageResult = Math.floor(answeredCorrectTotal / questionsTotal * 100);

  quizResultPercentageElement.append("Rezultat: " + percentageResult + "%");
  quizResultQuestionNumberElement.append("Imaš " + answeredCorrectTotal + " od " + questionsTotal + " točnih odgovora.");
  if (percentageResult < 51) {
    quizResultQuestionNumberElement.append("<br /> Ovaj puta nisi uspio. Ponovi gradivo i pokušaj ponovno.");
  } else if (percentageResult < 76) {
    quizResultQuestionNumberElement.append("<br /> Pokušaj ponovno! Ponovi gradivo i ostvari bolji rezultat.");
  } else {
    quizResultQuestionNumberElement.append("<br /> Bravo! Sada znaš: <br />");
    let outcomes = document.getElementsByClassName("dos-overview")[0].getElementsByTagName("UL")[0].getElementsByTagName("LI");
    let outcomesList = Array.prototype.slice.call(outcomes);
    outcomesList.forEach(element => quizResultQuestionNumberElement.append(element.innerText + "<br/>"));
  }

  quizResultElement.css("display", "inline-block");
  quizResultCheckButtonElement.css("display", "none");
};

const resetQuizResult = (quizId) => {
  const quizResultElement = getQuizResultElement(quizId);
  const quizResultCheckButtonElement = getQuizResultCheckButtonElement(quizId);
  const quizResultPercentageElement = getQuizResultPercentageElement(quizId);
  const quizResultQuestionNumberElement = getQuizResultQuestionNumberElement(quizId);

  quizResultPercentageElement.empty();
  quizResultQuestionNumberElement.empty();
  quizResultElement.css("display", "none");
  quizResultCheckButtonElement.css("display", "inline-block");
};

const signalizeQuestionCorrectness = (questionElement, isAnsweredCorrectly) => {
  removeQuestionCorectnessSignalization(questionElement);

  const answeredCorrectMap = getQuestionValidation(questionElement);

  answeredCorrectMap.map(answer => {
    let correctImg;
    let incorrectImg;
    if (questionElement.getAttribute('data-tasktype') === 'wordGroup' || questionElement.getAttribute('data-tasktype') === 'wordDrag') {
      if (isPreview) {
        correctImg = '<img class="correctAnswer" src="/explain/static/preview/1.0/img/check_mark.png" style="width:18px;height:18px;display:inline;" class="result-mark" alt="točan odgovor"/>';
        incorrectImg = '<img class="incorrectAnswer" src="/explain/static/preview/1.0/img/red_cross_mark.png" style="width:18px;height:18px;display:inline;" class="result-mark" alt="netočan odgovor"/>';
      } else {
        correctImg = '<img class="correctAnswer" src="../img/check_mark.png" style="width:18px;height:18px;display:inline;" class="result-mark" alt="točan odgovor"/>';
        incorrectImg = '<img class="incorrectAnswer" src="../img/red_cross_mark.png" style="width:18px;height:18px;display:inline;" class="result-mark" alt="netočan odgovor"/>';
      }
    } else {
      if (isPreview) {
        correctImg = '<img class="correctAnswer" src="/explain/static/preview/1.0/img/check_mark.png" style="width:24px;height:24px;display:inline;" class="result-mark" alt="točan odgovor"/>';
        incorrectImg = '<img class="incorrectAnswer" src="/explain/static/preview/1.0/img/red_cross_mark.png" style="width:24px;height:24px;display:inline;" class="result-mark" alt="netočan odgovor"/>';
      } else {
        correctImg = '<img class="correctAnswer" src="../img/check_mark.png" style="width:24px;height:24px;display:inline;" class="result-mark" alt="točan odgovor"/>';
        incorrectImg = '<img class="incorrectAnswer" src="../img/red_cross_mark.png" style="width:24px;height:24px;display:inline;" class="result-mark" alt="netočan odgovor"/>';
      }
    }

    const answerParentElement = answer.answer.parentElement;

    if (answer.answer.type === 'text' && answer.answer.tagName === 'INPUT') {
      const divForInputWithImage = $(`<div id="${answer.answer.id}" class="answerImgDiv"></div>`);
      if (answer.correct) {
        $(divForInputWithImage).insertBefore(answer.answer);
        $(answer.answer).appendTo(divForInputWithImage);
        addImagesToAnswers(answer, divForInputWithImage, null, null, null, null, correctImg);
      } else {
        $(divForInputWithImage).insertBefore(answer.answer);
        $(answer.answer).appendTo(divForInputWithImage);
        addImagesToAnswers(answer, divForInputWithImage, null, null, null, null, incorrectImg);
      }
    }

    let dragNDropElement;
    if (answer.answer.tagName === 'DIV') {
      dragNDropElement = answer.answer;
    }

    let selectElement;
    if (answerParentElement && answerParentElement.getElementsByTagName('select').length || answer.answer.tagName === 'SELECT') {
      if (answerParentElement && answerParentElement.getElementsByTagName('select')) {
        selectElement = answerParentElement.getElementsByTagName('select')[0];
      }

      if (answer.answer.tagName === 'SELECT') {
        selectElement = answer.answer;
      }
    }

    let labelElement;
    if (answerParentElement && answerParentElement.getElementsByTagName('label')) {
      labelElement = answerParentElement.getElementsByTagName('label')[0];
    }

    let listElement;
    if (answerParentElement && ['OL', 'UL', 'LI'].includes(answerParentElement.tagName)) {
      listElement = answerParentElement.tagName;
    }

    if (answer.correct) {
      addImagesToAnswers(answer, null, dragNDropElement, selectElement, labelElement, listElement, correctImg);
    } else {
      addImagesToAnswers(answer, null, dragNDropElement, selectElement, labelElement, listElement, incorrectImg);
    }
  });

  if (isAnsweredCorrectly) {
    setQuestionCorrectSignalization(questionElement);
  } else {
    setQuestionWrongSignalization(questionElement);
  }
};

const signalizeAnswerCorrectness = (answerElement, isAnsweredCorrectly) => {

  //removeAnswerCorectnessSignalization(answerElement);
  if (isAnsweredCorrectly) {
    //TBD
  } else {
    //TBD
  }
};

const removeQuestionCorectnessSignalization = (questionElement) => {
  const questionResultMarkElements = getQuestionResultMarkElements(questionElement);

  questionElement.classList.remove("bg-darkgray");
  questionElement.classList.remove("alert-danger");
  questionElement.classList.remove("alert-success");
  if(questionElement.getElementsByClassName("check-result").length == 0) $(questionElement).find(".odgovor-gumbi").prepend( "<div class='check-result'></div>" ); /*BT*/
  questionElement.getElementsByClassName("check-result")[0].innerHTML = "";
  questionResultMarkElements.remove();

  const correctImageElements = questionElement.getElementsByClassName('correctAnswer');
  for (var i = 0; i < correctImageElements.length; i++) {
    correctImageElements[i].style.display = 'none';
  }

  const incorrectImageElements = questionElement.getElementsByClassName('incorrectAnswer');
  for (var i = 0; i < incorrectImageElements.length; i++) {
    incorrectImageElements[i].style.display = 'none';
  }
};

const setQuestionCorrectSignalization = (questionElement) => {
  questionElement.classList.add("alert-success");
  questionElement.getElementsByClassName("check-result")[0].innerHTML = "";
  questionElement.getElementsByClassName("check-result")[0].innerHTML = "<span>Zadatak je u potpunosti točno riješen!</span>";
};

const setQuestionWrongSignalization = (questionElement) => {
  questionElement.classList.add("alert-danger");
  questionElement.getElementsByClassName("check-result")[0].innerHTML = "";
  questionElement.getElementsByClassName("check-result")[0].innerHTML = "<span>Zadatak nije u potpunosti točno riješen!</span>";
};

// ----- Initialize "drag elements" question elements
const initDragWordsElements = (questionElementId) => {
  var questionContextElement = questionElementId && document.getElementById(questionElementId) || document;
  let activeDragElement = null;

  function makeElementActive(el) {
    activeDragElement = el;
    $(el).addClass('dragging');
  }

  function removeActiveElement() {
    $(activeDragElement).removeClass('dragging');
    activeDragElement = null;
  }

  // make elements wider becase when wordGroup is evaluated elements change position
  $('.drop-polje, .ui-draggable, .ui-draggable-handle').css({
    'verticalAlign': 'middle'
  });

  // target dropzone elements
  $('.drop-polje, .bazen-odgovora', questionContextElement)
    .on('pointerdown', function(e) {
      if(activeDragElement) {
        handleDrop(activeDragElement, e.currentTarget, e);
      }
    })
    .droppable({
      revert: 'invalid',
      accept: '.drag-odgovor',
      tolerance: 'pointer',

      // listen for drop related events:

      over: (event, ui) => {
        var draggableElement = ui.draggable.get(0),
          dropzoneElement = event.target;

        // feedback the possibility of a drop
        dropzoneElement.classList.add('drop-polje-hover');
      },

      out: (event, ui) => {
        const draggableElement = ui.draggable.get(0);
        const dropzoneElement = event.target;

        dropzoneElement.classList.remove('drop-polje-hover');

        draggableElement.removeAttribute(dropTargetAttribute); //remove answer correctness mark
      },
      drop: (event, ui) => {
        const draggableElement = ui.draggable.get(0);
        const dropzoneElement = event.target;

        handleDrop(draggableElement, dropzoneElement, event);
      }
    }
  );

  function removeFromDropzone(draggable) {
    $(draggable).css({
      top: 0,
      left: 0,
    })
    .removeAttr('data-dragged-group-value')
    .removeAttr(dropTargetAttribute);
  }

  function handleDrop(draggableElement, dropzoneElement, event) {
    if ($(draggableElement).hasClass("dragging") && $(dropzoneElement).closest(".zadatak")[0].getAttribute('data-tasktype') !== questionTypeEnum.wordGroup) {
      $(draggableElement).removeClass("dragging");
    }

    /**
     * maybe in the future
     */
    // $(draggableElement).detach().css({top: 0,left: 0}).appendTo(dropzoneElement);

    if ($(draggableElement).closest(".zadatak")[0].getAttribute('data-tasktype') === 'wordDrag') {
      const taskId = `${$(draggableElement).closest(".zadatak")[0].getAttribute('id')}`;
      const divsByQuestionId = $(`div[id*='${taskId}']`);
      const dragCorrect = dropzoneElement.getAttribute(correctAnswerAttribute); //get answer data

      const dropzoneCurrentDraggableId = parseInt($(dropzoneElement).attr("id").split('-drop')[1]);
      const dropzoneQuestionId = $(dropzoneElement).attr("id").split('-drop')[0].toString();

      let currentDraggableInDropzone;
      let taskType;

      if (divsByQuestionId.length) {
        divsByQuestionId.toArray().filter(el => {
          if (el.getAttribute('data-tasktype') === 'wordDrag') {
            taskType = el.getAttribute('data-tasktype') === 'wordDrag';
          }
        });
      }

      if (dropzoneCurrentDraggableId !== undefined && taskType && dropzoneQuestionId === $(draggableElement).attr("id").split('-o')[0].toString()) {
        currentDraggableInDropzone = $('.drag-odgovor').toArray().filter(el => $(dropzoneElement).attr("id").split('-drop')[0].toString() === $(el).attr('id').split('-o')[0].toString() && parseInt($(el).attr('data-dragged-group-value')) === dropzoneCurrentDraggableId)[0];
      }

      if (draggableElement.classList.contains("drag-odgovor") && !dropzoneElement.classList.contains("bazen-odgovora") && dropzoneQuestionId === $(draggableElement).attr("id").split('-o')[0].toString()) {
        // snap draggable to dropzone
        $(draggableElement).position({
          of: $(dropzoneElement),
          my: 'center',
          at: 'middle'
        });
      }

      if (currentDraggableInDropzone && taskType && dropzoneQuestionId === $(draggableElement).attr("id").split('-o')[0].toString()) {
        removeFromDropzone(currentDraggableInDropzone);
      }

      draggableElement.setAttribute("data-dragged-group-value", event.target.getAttribute("id").split('-drop')[1]);
      draggableElement.setAttribute(dropTargetAttribute, dragCorrect); //set answer data to question
      
    } else {
      dropzoneElement.classList.remove('drop-polje-hover');

      if ($(draggableElement).closest(".zadatak")[0].getAttribute('data-tasktype') === questionTypeEnum.wordGroup) {
        $(draggableElement).detach().css({
          top: 0,
          left: 0
        }).appendTo(dropzoneElement);
        $($(dropzoneElement).parent()[0]).css({
          minHeight: '200px',
          height: 'auto',
        });
      }

      // append draggable to dropzone
      if ($(dropzoneElement).closest(".zadatak")[0].getAttribute('data-tasktype') === questionTypeEnum.wordGroup) {
        $(draggableElement).position({
          of: $(event),
          my: "left top",
          at: "left top",
        }).css({
          'position': 'relative',
          'left': '0px',
          'top': '0px',
          'float': 'left',
          'textAlign': 'center',
          'color': '#333333;',
        });
      }

      if (dropzoneElement.classList.contains("drop-pitanje")) {
        const dragCorrect = dropzoneElement.getAttribute(correctAnswerAttribute); //get answer data
        draggableElement.setAttribute(dropTargetAttribute, dragCorrect); //set answer data to question
        // snap draggable to dropzone
        $(draggableElement).position({
          of: $(dropzoneElement),
          my: 'center',
          at: 'middle'
        });
      }
      if (dropzoneElement.classList.contains("drop-grupa")) {
        const dragCorrect = dropzoneElement.getAttribute(correctAnswerAttribute); //get answer data
        draggableElement.setAttribute(dropTargetAttribute, dragCorrect); //set answer data to question
      }
    }
    removeActiveElement();
  };

  // target draggable elements

  $('.drag-odgovor', questionContextElement)
    .draggable({
      tolerance: 'pointer',
      revert: 'invalid',
      start: (event, ui) => {
      },

    })
    .each((index, element) => {
      $(element).draggable({
        containment: questionElementId ? getQuestionElement(questionElementId) : $(element).closest(".pitanje"), // keeping drag area inside some element
      });
    })
    .on('pointerdown', (e) => {
      const element = e.currentTarget;
      const isGrouped = $(element).attr('data-dragged-group-value') !== undefined;
        
      e.stopPropagation();

      if(activeDragElement) {
        if(isGrouped) {
          removeFromDropzone(element);
          return;
        }
        if (element === activeDragElement) {
          removeActiveElement();
        } else if (element !== activeDragElement) {
          removeActiveElement();
          makeElementActive(element);
        } else {
          console.warn('This should never happen');
          removeActiveElement();
        }
      } else {
        makeElementActive(element);
      }
    });
};

// ----- Initialize "sortable" question elements
const initSortElements = (questionElementId) => {
  let activeElement;
  let draggable;
  let droppable;

  $(getQuestionSortListElement(questionElementId).children()).droppable({
    tolerance: 'intersect',
    items: '> li:not(.draggable-disabled)',
    drop: function (event, ui) {
      droppable = event.target;
      draggable = ui.draggable.get(0);

      if ($(draggable).hasClass("dragging")) {
        $(draggable).removeClass("dragging");
      }

      if (!$(droppable).children().length || !$(droppable).children()[0].classList.contains("correctAnswer")) {
        replaceElements(droppable, draggable);
      }

      droppable = null;
      draggable = null;
    }
  });


  $(getQuestionSortListElement(questionElementId).children()).draggable({
      axis: 'y',
      items: '> li:not(.draggable-disabled)',
      cancel: "li.draggable-disabled",
      start: function (event, ui) {
        draggable = event.target;

        if (['sortList', 'pairList'].includes($(event.target)[0].closest(".zadatak").getAttribute('data-tasktype'))) {
          $('.draggable-disabled', this).each(function () {
            var $this = $(this);
            $this.data('pos', $this.index());
          });
        }
      },
      change: function (event, ui) {
        if (['sortList', 'pairList'].includes($(event.target)[0].get(0).closest(".zadatak").getAttribute('data-tasktype'))) {
          $sortable = $(this);
          $statics = $('.draggable-disabled', this).detach();
          $helper = $('<li></li>').prependTo(this);
          $statics.each(function () {
            var $this = $(this);
            var target = $this.data('pos');

            $this.insertAfter($('li', $sortable).eq(target));
          });
          $helper.remove();
        }
      },
      stop: function (event, ui) {
        draggable = event.target;

        event.target.style.left = "";
        event.target.style.top = "";

        // unmark active element
        event.target.classList.remove('draggable-active');
        event.target.classList.remove('can-drop-style');

        const questionId = questionElementId || $(event.target)[0].closest(".zadatak").id;
        setSortElementsOrderAttribute(questionId);

        if (['sortList', 'pairList'].includes($(event.target)[0].closest(".zadatak").getAttribute('data-tasktype'))) {
          getQuestionSortListElement(questionId);
          $('.draggable-disabled', this).each(function () {
            var $this = $(this);
            $this.data('pos', $this.index());
          });
        }
        removeActiveElement();
      },
      update: (event, ui) => {
        const questionId = questionElementId || $(event.target)[0].closest(".zadatak").id;
        if (['sortList', 'pairList'].includes($(event.target)[0].closest(".zadatak").getAttribute('data-tasktype'))) {
          setSortElementsOrderAttribute(questionId);
          getQuestionSortListElement(questionId);
        }
      },
    })
    .on('pointerdown', (e) => {
      const element = e.currentTarget;
      if(activeElement) {
        replaceElements(element, activeElement);
        removeActiveElement();
      } else {
        makeElementActive(element);
      }
    })
    .find('li')
    .each((index, element) => {
      // element.setAttribute(dropTargetAttribute, index);
      element.setAttribute(sortablePositionAttribute, index);
    })

    function makeElementActive (el) {
      activeElement = el;
      $(el).addClass('dragging');
    }

    function removeActiveElement() {
      $(activeElement).removeClass('dragging');
      activeElement = null;
    }

};

// ----- Utils -----

const shuffleArray = (array) => {
  const newArray = [...array];

  for (let i = array.length - 1; i > 0; i--) {
    let j = Math.floor(Math.random() * (i + 1)); // random index from 0 to i
    // swap elements array[i] and array[j]
    [newArray[i], newArray[j]] = [newArray[j], newArray[i]];
  }

  return newArray;
};

function replaceElements(droppable, draggable) {
  var dropzoneElementContent = droppable.innerHTML;
  var dropzoneElementCorrectPair = droppable.dataset.correct;

  // update dropzones content
  droppable.innerHTML = draggable.innerHTML;
  droppable.dataset.correct = draggable.dataset.correct;

  // update draggable's content
  draggable.innerHTML = dropzoneElementContent;
  draggable.dataset.correct = dropzoneElementCorrectPair;
  return;
}

// ----- Init quiz -----
const initQuiz = () => {
  initDragWordsElements();
  initSortElements();
};

initQuiz();


/*****
 -- -- -- -- -- -- -- -- -- --Something completely unrelated to quiz - logic-- -- -- -- -- -- -- -- -- --
 *****/

var focuseVid;
var allvids = $('video');

var focuseAudio;
var allauds = $('audio');

/** play only one video at a time **/
$("video").on("play", function (event) {
  focuseVid = event.target;
  var v_i = allvids.index(this);
  for (var i = 0; i < allvids.length; i++) {
    if (i == v_i) {
      allvids[v_i].play();
      for (var j = 0; j < allauds.length; j++) {
        allauds[j].pause();
      }
    } else {
      allvids[i].pause();
      for (var j = 0; j < allauds.length; j++) {
        allauds[j].pause();
      }
    }
  }
});

/** play only one audio at a time **/
$("audio").on("play", function (event) {
  focuseAudio = event.target;
  var a_i = allauds.index(this);
  for (var i = 0; i < allauds.length; i++) {
    if (i == a_i) {
      allauds[a_i].play();
      for (var j = 0; j < allvids.length; j++) {
        allvids[j].pause();
      }
    } else {
      allauds[i].pause();
      for (var j = 0; j < allvids.length; j++) {
        allvids[j].pause();
      }
    }
  }
});

$("video").on("click", function (e) {
  this.play();
});

$("audio").on("click", function (e) {
  this.play();
});


// ----- Set input elements to answer length
const initInputLength = (questionElementId) => {
/*
  const answerElementList = $("input[type='text']").toArray()
    .forEach((element) => {
      const correctAnswerLength = element.getAttribute(correctAnswerAttribute).length;
      switch (correctAnswerLength) {
        case 1:
        case 2:
          element.style.maxWidth = '3em';
          break;
        case 3:
        case 4:
        case 5:
          element.style.maxWidth = '6em';
          break;
        case 6:
        case 7:
        case 8:
        case 9:
          element.style.maxWidth = '11em';
          break;
        case 10:
        case 11:
        case 12:
        case 13:
        case 14:
          element.style.maxWidth = '16em';
          break;
        case 15:
        case 16:
        case 17:
        case 18:
        case 19:
          element.style.maxWidth = '21em';
          break;
        case 20:
        case 21:
        case 22:
        case 23:
        case 24:
          element.style.maxWidth = '26em';
          break;
        case 25:
        case 26:
        case 27:
        case 28:
        case 29:
          element.style.maxWidth = '31em';
          break;
        case 30:
        case 31:
        case 32:
        case 33:
        case 34:
          element.style.maxWidth = '36em';
          break;
        case 35:
        case 36:
        case 37:
        case 38:
        case 39:
          element.style.maxWidth = '41em';
          break;
        default:

      }

    });
*/
};

//initInputLength();
$(document).ready(function () {

  /***caption helper**/
  /*  if (typeof WebVttHelper === "function") {
      (new WebVttHelper).generateVTTCues()
    }
  */

  if ($("body").hasClass("italian")) {
    $(".main-navigation-dropdown>ul:first-child>li:first-child>a").text("Talijanski jezik 1");
    $(".skbreadcrumb a:first-child span").text("Talijanski jezik 1");
  } else if ($("body").hasClass("italian5")) {
    $(".main-navigation-dropdown>ul:first-child>li:first-child>a").text("Talijanski jezik 5");
    $(".skbreadcrumb a:first-child span").text("Talijanski jezik 5");

  };
});